package com.celebcam;

import android.graphics.Canvas;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.RectF;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.view.MotionEvent;
import android.view.View;
import android.content.Context;
import android.util.AttributeSet;
import android.util.Log;

public class CelebrityOverlayView extends View {
	
		static CelebrityOverlayView current;
		
		static Context mContext = null;

		static byte TRANSLATION = 0;
		static byte ROTATION    = 1;
		static byte SCALE       = 2;
		
	    private Bitmap  mCelebBitmap;
	    
	    private RectF   mBitmapCollider;
	    
	    private boolean mActive;
	    private int     mCurrentX;
	    private int     mCurrentY;
		private float[] mCenter;
		private float[] mInitialTouchPoints;
		private float[] mCurrentTouchPoints;
		
		private float   mInitialDistance;
	    private Matrix  mTransformationMatrix;
	    private byte    mTransformType;
	    
	    private float[] mBitmapCollider2;
	    
	    private boolean mShowManipulators;
	    
	    private Bitmap  mRotateManipulator;
	    private RectF   mRotateManipulatorRectF;
	    private Matrix  mManipulatorMatrix;
	    private float[] mCenterOfRotateManipulator;
	    private float   mRotateAmount;
	    
	    private Bitmap  moveManipulator;
	    private Bitmap  scaleManipulator;
	    
	    private static final byte CLOCKWISE = 0;
	    private static final byte COUNTER_CLOCKWISE = 1;
	    private static final byte NONE = -1;
	    private byte    rotateOrientation = NONE;
	    
	    private String text = " ", text2 = " ";
	    
		public CelebrityOverlayView(Context context, AttributeSet attriSet)
		{
			super( context, attriSet );
			
			current = this;
			mContext = context;
			
			mCelebBitmap    = BitmapFactory.decodeResource( getContext().getResources(), R.drawable.rihanna );
			
			mBitmapCollider = new RectF(0, 0, mCelebBitmap.getWidth(), mCelebBitmap.getHeight());

			mBitmapCollider2 = new float[] { 0, 0,
								 mCelebBitmap.getWidth(), 0,
								 0, mCelebBitmap.getHeight(),
								 mCelebBitmap.getWidth(), mCelebBitmap.getHeight() };
			
			mCenter = new float[]{ mCelebBitmap.getWidth()/2, mCelebBitmap.getHeight()/2 } ;
			
			mInitialTouchPoints = new float[4];
			mCurrentTouchPoints = new float[4];
			
			mTransformationMatrix = new Matrix();
			mTransformationMatrix.setScale((float)0.3, (float)0.3);		
			mTransformationMatrix.mapRect( mBitmapCollider);
			mTransformationMatrix.mapPoints(mBitmapCollider2);
			mTransformationMatrix.mapPoints(mCenter);
			

			mRotateManipulator = BitmapFactory.decodeResource( mContext.getResources(), R.drawable.rotate );
			
			mRotateManipulator = Bitmap.createScaledBitmap(mRotateManipulator, 50, 50, false);
			
			mRotateManipulatorRectF = new RectF( 0, 0, mRotateManipulator.getWidth(), mRotateManipulator.getHeight() );
			
			mCenterOfRotateManipulator = new float[]{ mRotateManipulator.getWidth()/2, mRotateManipulator.getHeight()/2 };
			
			mRotateAmount = mRotateManipulator.getWidth()/4;
			
			mManipulatorMatrix = new Matrix();

		}

		
		public void scale( float factor)
		{
		
			mTransformationMatrix.postScale(factor, factor );
			Matrix tmp = new Matrix();
			tmp.setScale( factor, factor );
			tmp.mapRect(mBitmapCollider);
			tmp.mapPoints(mBitmapCollider2);
			
			float[] prevCenter = new float[] { mCenter[0], mCenter[1] };
			
			tmp.mapPoints(mCenter);
			
			translate( prevCenter[0] - mCenter[0]   ,  prevCenter[1] - mCenter[1]);
			
			tmp = null;
		}
		
		public void translate( float dx, float dy )
		{
			mTransformationMatrix.postTranslate(dx, dy );
			mBitmapCollider.offset(dx, dy);
			
			Matrix tmp = new Matrix();
			tmp.setTranslate( dx, dy );
			tmp.mapPoints(mBitmapCollider2);
			tmp.mapPoints(mCenter);
			
			
			mManipulatorMatrix.setTranslate(mBitmapCollider2[0] - mRotateManipulator.getWidth(), mBitmapCollider2[1] - mRotateManipulator.getHeight());
			mRotateManipulatorRectF.offsetTo(mBitmapCollider2[0] - mRotateManipulator.getWidth(), mBitmapCollider2[1] - mRotateManipulator.getHeight());
			
			mCenterOfRotateManipulator[0] = mBitmapCollider2[0] - (mRotateManipulator.getWidth()/2);
			mCenterOfRotateManipulator[1] = mBitmapCollider2[1] - (mRotateManipulator.getHeight()/2);
			
			invalidate();
		}
		
		
		public void rotate( float degrees )
		{
			
			mTransformationMatrix.postRotate(degrees, mCenter[0], mCenter[1] );
			
			Matrix tmp = new Matrix();

			tmp.setRotate( degrees, mCenter[0], mCenter[1]);
			tmp.mapRect(mBitmapCollider);
			tmp.mapPoints(mBitmapCollider2);
			invalidate();
			tmp = null;
		}
		
		int d;
		
		public void zoom(int distance)
		{
			float a = 1;
			if( distance > d )
			{
				a = (float)1.1;
			}
			else if( distance < d )
			{
				a = (float)0.9;
			}
			
			d = distance;
			
			scale( a );
		}
		
		public void stopSmoothZoom(int distance)
		{
			
		}
		
		private void setCenter()
		{
			
		}
		private float leftMostX()
		{
			float minX = mBitmapCollider2[0];
			// 0,2,4,6
			for( int i = 0; i < 4; i++)
			{
				if( minX > mBitmapCollider2[i*2])
					minX = mBitmapCollider2[i*2];
			}
			
			return minX;
		}
		
		private float rightMostX()
		{
			float maxX = mBitmapCollider2[0];
			// 0,2,4,6
			for( int i = 0; i < 4; i++)
			{
				if( maxX < mBitmapCollider2[i*2])
					maxX = mBitmapCollider2[i*2];
			}
			
			
			return maxX;
		}
		
		private float topMostY()
		{
			float minY = mBitmapCollider2[1];
			// 1,3,5,7
			for( int i = 0; i < 4; i++)
			{
				if( minY > mBitmapCollider2[(i*2) + 1])
					minY = mBitmapCollider2[(i*2) + 1];
			}
			
			return minY;
		}
		
		private float bottomMostY()
		{
			float maxY = mBitmapCollider2[1];
			// 1,3,5,7
			for( int i = 0; i < 4; i++)
			{
				if( maxY < mBitmapCollider2[(i*2) + 1])
					maxY = mBitmapCollider2[(i*2) + 1];
			}
			
			return maxY;
		}
		
		private double distance(float[] array )
		{
			
			return Math.sqrt( Math.abs( array[0] - array[2] ) + Math.abs( array[1] - array[3] ) );
		}
		
		private boolean contains( float x, float y )
		{
			
			if( x >= leftMostX() && x <= rightMostX() && y >= topMostY() && y <= bottomMostY() )
				return true;
			
			return false;
		}
			
		String pc = new String();
		@Override
		public boolean onTouchEvent(MotionEvent event) {
			
				if( event.getAction() == MotionEvent.ACTION_DOWN )
				{
					mShowManipulators = !mShowManipulators;
					
					invalidate();
					
					if( contains( event.getX(), event.getY() ) )
					{
						Log.d("CelebView", "image touched");
						mActive = true;
						
						
						mCurrentX = (int)event.getX();
						mCurrentY = (int)event.getY();
						
						mTransformType = TRANSLATION;
						
					
					}
					else if( mRotateManipulatorRectF.contains( event.getX(), event.getY() ) )
					{
						Log.d("CelebView", "rotate");
						mActive = true;
						mShowManipulators = true;
						
						
						mInitialTouchPoints[0] = mCurrentX = (int)event.getX();
						mInitialTouchPoints[1] = mCurrentY = (int)event.getY();
						
	
						mTransformType = ROTATION;					
					}
				
				}
				else if( mActive && event.getAction() == MotionEvent.ACTION_MOVE )
				{
					if( mTransformType == TRANSLATION )
					{
						int xOffset, yOffset;
						
						xOffset = (int)event.getX() - mCurrentX;
				
						yOffset = (int)event.getY() - mCurrentY;
						
						mCurrentX = (int)event.getX();
						mCurrentY = (int)event.getY();
						
						translate(xOffset, yOffset);
					}
					else if( mTransformType == ROTATION && 
							 mRotateManipulatorRectF.contains( event.getX(), event.getY()))
					{
						if( event.getY() < mCenterOfRotateManipulator[1] )
						{
							if( event.getX() > mCurrentX )
							{
								rotate( mRotateAmount );
								mCurrentX = (int)event.getX();
							}
							else if( event.getX() < mCurrentX )
							{
								rotate( -mRotateAmount );
								mCurrentX = (int)event.getX();
							}
						}
						else if(event.getY() > mCenterOfRotateManipulator[1] )
						{
							if( event.getX() > mCurrentX )
							{
								rotate( -mRotateAmount );
								mCurrentX = (int)event.getX();
							}
							else if( event.getX() < mCurrentX )
							{
								rotate( mRotateAmount );
								mCurrentX = (int)event.getX();
							}
						}
					}
						
				}
				else if( event.getAction() == MotionEvent.ACTION_UP)
				{
					text = " ";
					mActive = false;
				}
			
			return mActive;
		}

		Paint paint = new Paint();
		public void onDraw(Canvas canvas )
		{
		super.layout( -10, -10, 500, 500 );
			canvas.drawColor(0x00000000);
			super.onDraw(canvas);
//			canvas.drawRect(mRotateManipulatorRectF, new Paint());
//			Log.d("CelebView", "calling onDraw");
			paint.setStrokeWidth(20);
			paint.setColor(0xffff3300);
			canvas.drawPoints(mBitmapCollider2, paint );
			canvas.drawBitmap( mCelebBitmap, mTransformationMatrix, null );
			canvas.drawPoints(mCenter, paint );

			
			if( mShowManipulators )
			{
				
				canvas.drawBitmap( mRotateManipulator, mManipulatorMatrix, null );
				canvas.drawPoints(mCenterOfRotateManipulator, paint );
			}

			canvas.drawText(text, 20, 200, paint);
			canvas.drawText(pc, 20, 300, paint);
		}
			
		public Bitmap getBitmap()
		{
		
			return Bitmap.createBitmap( mCelebBitmap, 0, 0, mCelebBitmap.getWidth(), mCelebBitmap.getHeight(), mTransformationMatrix, false );
		}
	}