package com.celebcam;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.hardware.Camera;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.os.Bundle;
import android.provider.MediaStore.Images;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.Toast;
import android.widget.ImageButton;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;

import com.celebcam.R;
import android.widget.GridView;
import android.widget.ScrollView;

public class DataAcquisitionActivity extends Activity implements SurfaceHolder.Callback {


	private String TAG = "CelebricamActivity";
	
    private Camera        mCamera;
    private SurfaceHolder mSurfaceHolder;
    private SurfaceView   mSurfaceView;
	
	byte[] tempdata;
	
	private Bitmap celebBitmap;
	private Bitmap finishedBitmap;
	private Bitmap takenPhotoBitmap;
	
	private Canvas mCanvas;
	
	ImageView mImageView;
	
	private ZoomAndSnapButton mZoomAndSnapButton;
	private CelebrityOverlayView mCelebView;
	private ScrollView        mCelebScrollView;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.data_acquisition);

        // View that camera preview is render on
        mSurfaceView = (SurfaceView) findViewById(R.id.camera_surface);
        mSurfaceHolder = mSurfaceView.getHolder();
        mSurfaceHolder.addCallback(this);
        mSurfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

        
        celebBitmap = BitmapFactory.decodeResource(this.getResources(), R.drawable.rihanna);
		
        finishedBitmap = Bitmap.createBitmap( 320,480, Bitmap.Config.ARGB_8888  );
        
        mCanvas = new Canvas( finishedBitmap );
        
        CCEffectsLibrary.setCanvas(mCanvas);
        
        GridView gridView = (GridView) findViewById( R.id.effects_palette );
        
        gridView.setAdapter(new ImageAdapter(this));
        gridView.setOnItemClickListener( new OnItemClickListener() {
        	public void onItemClick( AdapterView<?> parent, View v, int position, long id )
        	{
        		Toast.makeText( DataAcquisitionActivity.this, " " + position, Toast.LENGTH_SHORT).show();
        	}
        	
        });
        
        mCelebView = (CelebrityOverlayView) findViewById( R.id.celeb_surface );
        
        mZoomAndSnapButton = (ZoomAndSnapButton) findViewById(R.id.zoom_snap_button);
        
        mZoomAndSnapButton.setCelebView( mCelebView );
        mZoomAndSnapButton.setOnClickListener(new View.OnClickListener() {
        		public void onClick(View view )
        		{
        			mCamera.takePicture(mShutterCallback, mPictureCallback, mjpeg);
        		}
        });

    }
    
    
    public void surfaceCreated(SurfaceHolder holder) {
        // The Surface has been created, now tell the camera where to draw the preview.
        mCamera = Camera.open();
        mZoomAndSnapButton.setCamera( mCamera );
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // empty. Take care of releasing the Camera preview in your activity.
        mCamera.stopPreview();
        mCamera.release();
        mCamera = null;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // If your preview can change or rotate, take care of those events here.
        // Make sure to stop the preview before resizing or reformatting it.

        if (mSurfaceHolder.getSurface() == null){
          // preview surface does not exist
          return;
        }

        // stop preview before making changes
        try {
            mCamera.stopPreview();
        } catch (Exception e){
          // ignore: tried to stop a non-existent preview
        }

        // set preview size and make any resize, rotate or
        // reformatting changes here

        // start preview with new settings
        try {
        Camera.Parameters p = mCamera.getParameters();
        p.setPreviewSize(w, h);
            mCamera.setPreviewDisplay(holder);
            mCamera.startPreview();

        } catch (Exception e){
        
        Log.d("ERROR", "Error starting camera preview: " + e.getMessage());
        }
    }
    
//    /** Check if this device has a camera */
//	private boolean checkCameraHardware(Context context) {
//	    if (context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA)){
//	        // this device has a camera
//	        return true;
//	    } else {
//	        // no camera on this device
//	        return false;
//	    }
//	}
	
	public static Camera getCameraInstance(){
	    Camera c = null;
	    try {
	        c = Camera.open(); // attempt to get a Camera instance
	    }
	    catch (Exception e){
	        // Camera is not available (in use or does not exist)
	    }
	    return c; // returns null if camera is unavailable
	}

	ShutterCallback mShutterCallback = new ShutterCallback(){
		public void onShutter() {}
	};
	
	PictureCallback mPictureCallback = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera c) {}
	};
	
	PictureCallback mjpeg = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera camera) {
			if( data != null ) {
				tempdata = data;
				done();
			}
		}
	};

	void done() {
		Bitmap bm = BitmapFactory.decodeByteArray( tempdata, 0, tempdata.length);
		
		takenPhotoBitmap = bm;
		
		takenPhotoBitmap = Bitmap.createScaledBitmap(takenPhotoBitmap, 320, 480, false);
		
		CCEffectsLibrary.composeImage( takenPhotoBitmap, CelebrityOverlayView.current.getBitmap() );
		
		String url = Images.Media.insertImage( getContentResolver(), finishedBitmap, null, null );
		
		//bm.recycle();
		
		Bundle bundle = new Bundle();
		
		if( url != null ) {
			bundle.putString("url", url);
			
			Intent mIntent = new Intent();
			mIntent.putExtras(bundle);
			setResult(RESULT_OK, mIntent);
		}else{
			Toast.makeText( this, "Picture can not be saved", Toast.LENGTH_SHORT).show();
		}
		
		//finish();
		mCamera.startPreview();
	}
		
}